pub fn min_plus_convolution_convex_divide_and_conquer<T>(
a: &[T],
b: &[T],
) -> Vec<T>where
T: Signed,Expand description
Computes convolution when one input is convex using monotone divide and conquer.
Panics
Panics unless both inputs are finite and at least one is convex.